import 'dart:math';
import 'dart:typed_data';
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:google_mlkit_text_recognition/google_mlkit_text_recognition.dart';
import 'package:logging/logging.dart';

/// Attempt to extract total amount from invoice image
Future<double?> extractTotalFromBill(Uint8List imgBuff) async {
  final decodedImage = await decodeImageFromList(imgBuff);

  final byteData = await decodedImage.toByteData(
    format: ui.ImageByteFormat.rawRgba,
  );

  final image = InputImage.fromBitmap(
    bitmap: byteData!.buffer.asUint8List(),
    width: decodedImage.width,
    height: decodedImage.height,
  );

  final textRecognizer = TextRecognizer(script: TextRecognitionScript.latin);
  final extractionResult = await textRecognizer.processImage(image);

  Logger.root.fine("Expense text: ${extractionResult.text}");

  // Check for highest amount on invoice
  final regexp = RegExp(
    r'([0-9]+([ ]*(\\.|,)[ ]*[0-9]{1,2}){0,1})[ \\t\\n]*(EUR|eur|€)',
    multiLine: true,
    caseSensitive: false,
  );
  var highest = 0.0;
  for (final match in regexp.allMatches(extractionResult.text)) {
    if (match.groupCount == 0) continue;

    // Process only numeric value
    final value = (match.group(1) ?? "").replaceAll(",", ".");
    highest = max(highest, double.tryParse(value) ?? 0.0);
  }

  return highest == 0.0 ? null : highest;
}
